#
# LUOS CLI tool

# *******************************************************************************
# Import packages
# *******************************************************************************
import argparse
import sys
import time
from multiprocessing import Process, Value
import json
from pyluos import Device
import numpy as np
import math
import crc8

# *******************************************************************************
# Global Variables
# *******************************************************************************
BOOTLOADER_IDLE = 0
BOOTLOADER_START = 1
BOOTLOADER_STOP = 2
BOOTLOADER_READY = 3
BOOTLOADER_ERASE = 4
BOOTLOADER_BIN_CHUNK = 5
BOOTLOADER_BIN_END = 6
BOOTLOADER_CRC_TEST = 7
BOOTLOADER_READY_RESP = 16
BOOTLOADER_BIN_HEADER_RESP = 17
BOOTLOADER_ERASE_RESP = 18
BOOTLOADER_BIN_CHUNK_RESP = 19
BOOTLOADER_BIN_END_RESP = 20
BOOTLOADER_CRC_RESP = 21
BOOTLOADER_ERROR_SIZE = 32

FILEPATH = None
NB_SAMPLE_BY_FRAME_MAX = 127

RESP_TIMEOUT = 3
ERASE_TIMEOUT = 10
PROGRAM_TIMEOUT = 5
# *******************************************************************************
# Function
# *******************************************************************************

# *******************************************************************************
# @brief find routing table
# @param port connected to the luos gate
# @return an object containing routing table
# *******************************************************************************
def find_network(device):
    device._send({'detection': {}})
    startTime = time.time()
    state = device._poll_once()
    while ('routing_table' not in state):
        if ('route_table' in state):
            print('version of luos not supported')
            return
        state = device._poll_once()
        if (time.time()-startTime > 1):
            device._send({'detection': {}})
            startTime = time.time()

    return state

# *******************************************************************************
# @brief find nodes to program in the network
# @param target list, routing table
# @return a tuple with 2 lists : nodes to reboot and nodes to program
# *******************************************************************************
def create_target_list(args, state):
    bypass_node = False
    nodes_to_program = []
    nodes_to_reboot = []
    for node in state['routing_table']:
        # bypass if node contains a Gate service
        # prevent programmation of a Gate
        for service in node['services']:
            bypass_node = False
            if(service['type'] == 'Gate'):
                bypass_node = True
                break
            if(service['alias'] == 'Pipe_mod'):
                bypass_node = True
        # check if node is in target list
        if not (bypass_node):
            nodes_to_reboot.append(node['node_id'])
            for target in args.target:
                if(int(node['node_id']) == int(target)):
                    nodes_to_program.append(node['node_id'])

    return (nodes_to_reboot, nodes_to_program)

# *******************************************************************************
# @brief send commands
# @param command type
# @return None
# *******************************************************************************
def send_command(device, node, command, size = 0):
    # create a json file with the list of the nodes to program
    bootloader_cmd = {
        'bootloader': {
            'command': {
                'type': command,
                'node': node,
                'size': size
            },
        }
    }
    # send json command
    device._send(bootloader_cmd)

# *******************************************************************************
# @brief get binary size
# @param
# @return binary size
# *******************************************************************************
def get_binary_size():
    # get number of bytes in binary file
    with open(FILEPATH, mode="rb") as f:
        nb_bytes = len(f.read())

    return nb_bytes

# *******************************************************************************
# @brief send erase command
# @param command type
# @return None
# *******************************************************************************
def send_ready_cmd(device, node):
    return_value = True

    # send ready command to the node
    send_command(device, node, BOOTLOADER_READY, get_binary_size())

    # wait ready response
    state = device._poll_once()
    init_time = time.time()
    while ('bootloader' not in state):
        state = device._poll_once()
        if(time.time() - init_time > RESP_TIMEOUT):
            print("  ╰> Node n°", node, "is not responding.")
            print("  ╰> Loading program aborted, please reboot the system.")
            return_value = False
            break
    if (state['bootloader']['response'] == BOOTLOADER_ERROR_SIZE):
        print("  ╰> Node n°", node, "has not enough space in flash memory.")
        # don't load binary if there is not enough place in flash memory
        return_value = False
    if (state['bootloader']['response'] == BOOTLOADER_READY_RESP):
        print("  ╰> Node n°", node, "is ready.")

    return return_value
# *******************************************************************************
# @brief waiting for erase response
# @param
# @return binary size
# *******************************************************************************
def waiting_erase():
    init_time = time.time()
    count = 0
    period = 0.4
    print("\r                        ", end='')
    print("\r  ╰> Erase flash        ", end='')
    while(1):
        if(time.time() - init_time > period):
            init_time = time.time()
            if(count == 0):
                print("\r                        ", end='')
                print("\r  ╰> Erase flash .      ", end='')
                count += 1
                continue
            if(count == 1):
                print("\r                        ", end='')
                print("\r  ╰> Erase flash ..     ", end='')
                count += 1
                continue
            if(count == 2):
                print("\r                        ", end='')
                print("\r  ╰> Erase flash ...    ", end='')
                count += 1
                continue
            if(count == 3):
                print("\r                        ", end='')
                print("\r  ╰> Erase flash        ", end='')
                count = 0
                continue

# *******************************************************************************
# @brief send erase command
# @param command type
# @return None
# *******************************************************************************
def erase_flash(device, node):
    return_value = True

    # send erase command
    send_command(device, node, BOOTLOADER_ERASE)

    # display a progress bar
    waiting_bg = Process(target=waiting_erase)
    waiting_bg.start()

    # wait ready response
    state = device._poll_once()
    init_time = time.time()
    while ('bootloader' not in state):
        state = device._poll_once()
        if(time.time() - init_time > ERASE_TIMEOUT):
            print("  ╰> Node n°", node, "is not responding.")
            print("  ╰> Loading program aborted, please reboot the system.")
            return_value = False
            break
    if (state['bootloader']['response'] == BOOTLOADER_ERASE_RESP):
        waiting_bg.terminate()
        print("\r\n  ╰> Flash memory erased.")

    return return_value

# *******************************************************************************
# @brief get binary size
# @param
# @return binary size
# *******************************************************************************
def loading_bar(loading_progress):
    init_time = time.time()
    period = 0.2
    while(1):
        if(time.time() - init_time > period):
            init_time = time.time()
            print("\r  ╰> loading : {} %".format(loading_progress.value), end='')

# *******************************************************************************
# @brief send the binary file to the node
# @param command type
# @return None
# *******************************************************************************
def send_binary_data(device, node):
    loading_state = True
    # compute total number of bytes to send
    with open(FILEPATH, mode="rb") as f:
        nb_bytes = len(f.read())
    # compute total number of frames to send
    nb_frames = math.ceil(nb_bytes / NB_SAMPLE_BY_FRAME_MAX)

    # display a progress bar to inform user
    loading_progress = Value('f', 0.0)
    loading_bar_bg = Process(target=loading_bar, args=(loading_progress,))
    loading_bar_bg.start()

    # send each frame to the network
    file_offset = 0
    for frame_index in range(nb_frames):
        if (frame_index == (nb_frames-1)):
            # last frame, compute size
            frame_size = nb_bytes - (nb_frames-1)*NB_SAMPLE_BY_FRAME_MAX
        else:
            frame_size = NB_SAMPLE_BY_FRAME_MAX

        # send the current frame
        loading_state = send_frame_from_binary(device, node, frame_size, file_offset)
        if( loading_state != True):
            break
        # update cursor position in the binary file
        file_offset += frame_size
        # update loading progress
        loading_progress.value = math.trunc(frame_index / nb_frames * 100)

    # kill the progress bar at the end of the loading
    loading_bar_bg.terminate()
    if( loading_state == True):
        print("\r  ╰> loading : 100.0 %")

    return loading_state

# *******************************************************************************
# @brief open binary file and send a frame
# @param
# @return None
# *******************************************************************************
def send_frame_from_binary(device, node, frame_size, file_offset):
    return_value = True

    with open(FILEPATH, mode="rb") as f:
        # put the cursor at the beginning of the file
        f.seek(file_offset)
        # read binary data
        data_bytes = f.read(1)
        for sample in range(frame_size):
            data_bytes = data_bytes + f.read(1)

    send_data(device, node, BOOTLOADER_BIN_CHUNK, frame_size, data_bytes)

    # wait node response
    lock = 1
    state = device._poll_once()
    init_time = time.time()
    while (lock):
        if(time.time() - init_time > PROGRAM_TIMEOUT):
            print("\r\n  ╰> Node n°", node, "is not responding.")
            print("  ╰> Loading program aborted, please reboot the system.")
            return_value = False
            break
        if 'bootloader' in state:
            if (state['bootloader']['response'] == BOOTLOADER_BIN_CHUNK_RESP):
                lock = 0
        state = device._poll_once()

    return return_value

# *******************************************************************************
# @brief send binary data with a header
# @param
# @return None
# *******************************************************************************
def send_data(device, node, command, size, data):
    # create a json file with the list of the nodes to program
    bootloader_cmd = {
        'bootloader': {
            'command': {
                'type': command,
                'node': node,
                'size': size
            },
        }
    }
    # send json command
    device._write(json.dumps(bootloader_cmd).encode() + '\r'.encode() + data)

# *******************************************************************************
# @brief send the binary end command
# @param
# @return
# *******************************************************************************
def send_binary_end(device, node):
    return_value = True

    # send command
    send_command(device, node, BOOTLOADER_BIN_END)
    # wait bin_end response
    state = device._poll_once()
    init_time = time.time()
    while ('bootloader' not in state):
        state = device._poll_once()
        if(time.time() - init_time > RESP_TIMEOUT):
            print("  ╰> Node n°", node, "is not responding.")
            print("  ╰> Loading program aborted, please reboot the system.")
            return_value = False
            break
    if (state['bootloader']['response'] == BOOTLOADER_BIN_END_RESP):
        print("  ╰> Node acknowledge received, loading is complete.")

    return return_value

# *******************************************************************************
# @brief compute binary crc
# @param
# @return None
# *******************************************************************************
def compute_crc():
    # create crc8 function object
    hash = crc8.crc8()
    # get number of bytes in binary file
    with open(FILEPATH, mode="rb") as f:
        nb_bytes = len(f.read())

    with open(FILEPATH, mode="rb") as f:
        for bytes in range(nb_bytes):
            data = f.read(1)
            hash.update(data)
            crc = hash.digest()

    return crc

# *******************************************************************************
# @brief send the binary end command
# @param
# @return
# *******************************************************************************
def check_crc(device, node):
    return_value = True

    # send crc command
    send_command(device, node, BOOTLOADER_CRC_TEST)
    # wait bin_end response
    state = device._poll_once()
    init_time = time.time()
    while ('bootloader' not in state):
        state = device._poll_once()
        if(time.time() - init_time > RESP_TIMEOUT):
            print("  ╰> Node n°", node, "is not responding.")
            print("  ╰> Loading program aborted, please reboot the system.")
            return_value = False
            break
    if (state['bootloader']['response'] == BOOTLOADER_CRC_RESP):
        source_crc = int.from_bytes(compute_crc(), byteorder='big')
        node_crc = state['bootloader']['crc_value']
        if ( source_crc == node_crc ):
            print("  ╰> CRC test : OK.")
        else:
            print("  ╰> CRC test : NOK.")
            print("  ╰> waited :", hex(source_crc), ", received :", hex(node_crc))
            return_value = False
    else:
        print("  ╰> CRC note received.")
        return_value = False

    return return_value

# *******************************************************************************
# @brief reboot all nodes in application mode
# @param
# @return
# *******************************************************************************
def reboot_network(device, nodes_to_reboot):
    for node in nodes_to_reboot:
                send_command(device, node, BOOTLOADER_STOP)
                # delay to let gate send commands
                time.sleep(0.01)
# *******************************************************************************
# @brief command used to flash luos nodes
# @param flash function arguments : -g, -t, -b
# @return None
# *******************************************************************************
def luos_flash(args):
    print('Luos flash subcommand with parameters :')
    print('\t--gate : ', args.gate)
    print('\t--target : ', args.target)
    print('\t--binary : ', args.binary)
    print('\t--port : ', args.port)

    if not (args.port):
        print('Please specify a port to access the network.')
        sys.exit()

    # state used to check each step
    machine_state = True

    # update firmware path
    global FILEPATH
    FILEPATH = args.binary
    try:
        f = open(FILEPATH, mode="rb")
    except IOError:
        print("Cannot open :", FILEPATH)
        sys.exit()
    else:
        f.close()

    # init device
    device = Device(args.port, background_task=False)

    # find routing table
    state = find_network(device)

    # searching nodes to program in network
    (nodes_to_reboot, nodes_to_program) = create_target_list(args, state)

    # reboot all nodes in bootloader mode
    print("** Reboot all nodes in bootloader mode **")
    for node in nodes_to_reboot:
        send_command(device, node, BOOTLOADER_START)
        # delay to let gate send commands
        time.sleep(0.01)

    # wait before next step
    time.sleep(0.1)

    # program nodes
    for node in nodes_to_program:
        print("** Programming node n°", node, "**")

        # go to header state if node is ready
        print("--> Check if node n°", node, "is ready.")
        machine_state = send_ready_cmd(device, node)
        if( machine_state != True):
            break

        # erase node flash memory
        print("--> Erase flash memory.")
        machine_state = erase_flash(device, node)
        if( machine_state != True):
            break

        # send binary data
        print("--> Send binary data.")
        machine_state = send_binary_data(device, node)
        if( machine_state != True):
            break

        # inform the node of the end of the loading
        print("--> Programmation finished, waiting for acknowledge.")
        machine_state = send_binary_end(device, node)
        if( machine_state != True):
            break

        # Ask the node to send binary crc
        print("--> Check binary CRC.")
        machine_state = check_crc(device, node)
        if( machine_state != True):
            break

    # wait before next step
    time.sleep(0.1)

    # reboot all nodes in application mode
    if (machine_state == True):
        print("** Reboot all nodes in application mode **")
        reboot_network(device, nodes_to_reboot)


# *******************************************************************************
# @brief command used to detect network
# @param detect function arguments : -p
# @return None
# *******************************************************************************
def luos_detect(args):
    print('Luos detect subcommand on port : ', args.port)

    if not (args.port):
        print('Please specify a port to access the network.')
        sys.exit()

    # detect network
    device = Device(args.port)
    # print network to user
    print(device.nodes)

# *******************************************************************************
# @brief command used to detect network
# @param detect function arguments : -p
# @return None
# *******************************************************************************
def luos_options():
    parser = argparse.ArgumentParser(
        description='Luos command line interface')

    # create subcommands
    subparsers = parser.add_subparsers()

    # declare "flash" subcommand
    flash_parser = subparsers.add_parser('flash',
                                         help='tool to program luos nodes')
    flash_parser.add_argument('port', help='port used to detect network')
    flash_parser.add_argument('-g', '--gate',
                              help='id of the gate used to access the luos network')
    flash_parser.add_argument('-b', '--binary',
                              help='path to the binary file to flash')
    flash_parser.add_argument('-t', '--target',
                              help='target node to flash',
                              default=['2'],
                              nargs='*')
    flash_parser.set_defaults(func=luos_flash)

    # declare "detect" subcommand
    detect_parser = subparsers.add_parser('detect',
                                          help='tool to detect luos network')
    detect_parser.add_argument('port', help='port used to detect network')
    detect_parser.set_defaults(func=luos_detect)

    return parser

# *******************************************************************************
# @brief parse arguments and launch subcommand function
# @param None
# @return None
# *******************************************************************************
def main():
    # declare options of the CLI
    parser = luos_options()

    # parse options
    args = parser.parse_args()

    # execute CLI subcommand
    args.func(args)

    return 0


if __name__ == '__main__':
    sys.exit(main())



"""
Pyluos command line utility

"""

from __future__ import print_function
import sys

import sys
import os
import argparse
from IPython import embed

import pyluos
from pyluos import Device
from pyluos.tools.discover import serial_discover


# *******************************************************************************
# @brief parse arguments and launch subcommand function
# @param None
# @return None
# *******************************************************************************
def main():

    ## Parse arguments ##
    parser = argparse.ArgumentParser(description='Luos command line utility based on pyluos\n',
                                     formatter_class=argparse.RawTextHelpFormatter)

    # General arguments
    parser.add_argument("-p", "--port", metavar="PORT", action="store",
                        help="The port where a Luos device should be discovered.\n"
                        "This can be a serial interface or an IP adress.\n"
                        "For example to specify a serial port : luostool --port \"cu.usbserial-DN2YEFLN\"",
                        default=None)

    parser.add_argument("--version", action="store_true",
                        help="print version information and exit")

    args = parser.parse_args()

    def print_version():
        sys.stderr.write("luos control utility v" + pyluos.__version__ + "\n")
        sys.stderr.flush()
        return 0

    try:
        if args.version == True:
            print_version()

        if args.port is not None:
            # Ready to rocks
            device = Device(args.port)
            embed(banner1 = "\n Hit Ctrl-D to exit this interpreter.\n\nYour luos device have been successfully mounted into a \"device\" object:\n" + str(device.nodes) + "\n")
        else:
            Gates = serial_discover()
            if Gates:
                device = Device(Gates[0])
                embed(banner1 = "\n Hit Ctrl-D to exit this interpreter.\n\nYour luos device have been successfully mounted into a \"device\" object:\n" + str(device.nodes) + "\n")

    except OperationAbortedException:
        logger.info("Operation aborted.")

    return 0


if __name__ == '__main__':
    sys.exit(main())



from __future__ import division

import json
import time
import serial as _serial
import platform
import sys
if sys.version_info >= (3, 0):
    import queue
else:
    import Queue as queue


from threading import Event, Thread

from serial.tools.list_ports import comports

from . import IOHandler

try:
    JSONDecodeError = json.decoder.JSONDecodeError
except AttributeError:
    JSONDecodeError = ValueError

class Serial(IOHandler):
    poll_frequency = 200

    @classmethod
    def available_hosts(cls):
        devices = comports(include_links=True)

        return [d.device for d in devices]

    @classmethod
    def is_host_compatible(cls, host):
        return host in cls.available_hosts()

    def __init__(self, host, baudrate=1000000):
        self._serial = _serial.Serial(host, baudrate)
        self._serial.flush()

        self._msg = queue.Queue(100)
        self._running = True

        self._poll_loop = Thread(target=self._poll)
        self._poll_loop.daemon = True
        self._poll_loop.start()

    def is_ready(self):
        if self._serial.in_waiting == 0:
            return False

        try:
            self.read()
            return True
        except (UnicodeDecodeError, JSONDecodeError):
            return False

    def recv(self):
        return self._msg.get()

    def write(self, data):
        self._serial.write(data + '\r'.encode())
        #print(data + '\r'.encode())

    def close(self):
        self._running = False
        self._poll_loop.join()

        self._serial.close()

    def _poll(self):
        def extract_line(s):
            j = s.find(b'\n')
            if j == -1:
                return b'', s
            # Sometimes the begin of serial data can be wrong remove it
            # Find the first '{'

            x = s.find(b'{')
            if x == -1:
                return b'', s[j + 1:]

            return s[x:j], s[j + 1:]

        period = 1 / self.poll_frequency
        buff = b''

        while self._running:
            to_read = self._serial.in_waiting

            if to_read == 0:
                time.sleep(period)
                continue

            s = self._serial.read(to_read)
            buff = buff + s

            while self._running:
                line, buff = extract_line(buff)
                if not len(line):
                    break
                if self._msg.full():
                    self._msg.get()
                self._msg.put(line)



from .service import Service, interact
import numpy as np


class Color(Service):
    def __init__(self, id, alias, device):
        Service.__init__(self, 'Color', id, alias, device)
        self._time = None
        self._size = None
        self._color = None

    @property
    def color(self):
        return self._color

    @color.setter
    def color(self, new_color):
        new_color = [int(min(max(c, 0), 255)) for c in new_color]
        if len(new_color) > 3 :
            self._color = new_color
            self._push_data('color', [len(new_color)], np.array(new_color, dtype=np.uint8))
        else :
            self._color = new_color
            self._push_value('color', new_color)
    @property
    def time(self):
        return self._time

    @time.setter
    def time(self, new_time):
        self._time = new_time
        self._push_value('time', new_time)

    @property
    def size(self):
        return self._size

    @size.setter
    def size(self, new_size):
        self._size = new_size
        self._push_value('parameters', new_size)

    def _update(self, new_state):
        Service._update(self, new_state)

    def control(self):
        def change_color(red, green, blue):
            self.color = (red, green, blue)

        return interact(change_color,
                        red=(0, 255, 1),
                        green=(0, 255, 1),
                        blue=(0, 255, 1))



from threading import Thread

from SimpleWebSocketServer import WebSocket, SimpleWebSocketServer

from pyluos.io.serial_io import Serial


class SerialToWs(WebSocket):
    def handleConnected(self):
        print('Connection opened.')
        self.serial = Serial(self.serial_port)

        self.t = Thread(target=self._check_msg)
        self.t.daemon = True
        self.t.start()

    def handleClose(self):
        if self.verbose:
            print('Connection closed.')
        self.serial.close()

    def handleMessage(self):
        message = self.data

        if self.verbose:
            print('WS->Ser: {}'.format(message))

        try:
            message = message.encode()
            self.serial.write(message)

        except UnicodeDecodeError:
            print("Fail")
            pass


    def send(self, message):
        if self.verbose:
            print('Ser->WS: {}'.format(message))
        self.sendMessage(message)

    def _check_msg(self):
        while True:
            r = self.serial.recv()
            try:
                self.send(r)
            except UnicodeDecodeError:
                print('Fail', r)

        print('LOOP OVER!')


def main():
    import argparse

    parser = argparse.ArgumentParser()
    parser.add_argument('--serial-port', type=str, required=True)
    parser.add_argument('--ws-port', type=int, required=True)
    parser.add_argument('--verbose', action='store_true', default=False)
    args = parser.parse_args()

    SerialToWs.serial_port = args.serial_port
    SerialToWs.verbose = args.verbose

    io_server = SimpleWebSocketServer('', args.ws_port, SerialToWs)
    io_server.serveforever()


if __name__ == '__main__':
    main()



from .service import Service


class Unknown(Service):
    possible_events = {'changed', 'pressed', 'released'}

    # control modes
    _PLAY = 0
    _PAUSE = 1
    _STOP = 2
    _REC = 4

    def __init__(self, id, alias, device):
        Service.__init__(self, 'Unknown', id, alias, device)
        self._control = 0
        self._state = False
        self._angular_position = 0.0
        self._angular_speed = 0.0
        self._trans_position = 0.0
        self._trans_speed = 0.0
        self._current = 0.0
        self._temperature = 0.0
        self._color = [0, 0, 0]
        self._time = 0.0
        self._parameters = 0
        self._pid = [0, 0, 0]
        self._power_ratio = 0.0
        self._lux = 0.0
        self._load = 0.0
        self._volt = 0.0

    def _update(self, new_state):
        Service._update(self, new_state)
        if 'io_state' in new_state:
            val = new_state['io_state']
            if val != self._state:
                self._pub_event('changed', self._state, val)

                evt = 'pressed' if val == True else 'released'
                self._pub_event(evt, self._state, val)

                self._state = val
        if 'rot_position' in new_state:
            self._angular_position = new_state['rot_position']
        if 'rot_speed' in new_state:
            self._angular_speed = new_state['rot_speed']
        if 'trans_position' in new_state:
            self._trans_position = new_state['trans_position']
        if 'trans_speed' in new_state:
            self._trans_speed = new_state['trans_speed']
        if 'current' in new_state:
            self._current = new_state['current']
        if 'temperature' in new_state:
            self._temperature = new_state['temperature']
        if 'lux' in new_state:
            self._lux = new_state['lux']
        if 'force' in new_state:
            self._load = new_state['force']
        if 'volt' in new_state:
            self._volt = new_state['volt']



    def play(self):
        if (self._control >= self._REC):
            self._control = self._PLAY + self._REC
        else :
            self._control = self._PLAY
        self._push_value('control', self._control)

    def pause(self):
        if (self._control >= self._REC):
            self._control = self._PAUSE + self._REC
        else :
            self._control = self._PAUSE
        self._push_value('control', self._control)

    def stop(self):
        # also stop recording
        self._control = self._STOP
        self._push_value('control', self._control)

    def rec(self, enable):
        if (self._control >= self._REC):
            if (enable == False):
                self._control = self._control - self._REC
        else :
            if (enable == True):
                self._control = self._control + self._REC
        self._push_value('control', self._control)

    @property
    def state(self):
        return self._state == True

    @state.setter
    def state(self, new_val):
        self._state = new_val
        self._push_value('io_state', new_val)

    @property
    def angular_position(self):
        """ Position in degrees. """
        return self._angular_position

    @angular_position.setter
    def angular_position(self, new_val):
        self._angular_position == new_val
        self._push_value('target_rot_position', new_val)

    @property
    def angular_speed(self):
        return self._angular_speed

    @angular_speed.setter
    def angular_speed(self, s):
        self._angular_speed = s
        self._push_value("target_rot_speed", s)

    @property
    def translation_position(self):
        """ Position in degrees. """
        return self._trans_position

    @translation_position.setter
    def translation_position(self, new_val):
        self._trans_position == new_val
        self._push_value('target_trans_position', new_val)

    @property
    def translation_speed(self):
        return self._angular_speed

    @angular_speed.setter
    def translation_speed(self, s):
        self._angular_speed = s
        self._push_value("target_rot_speed", s)

    @property
    def current(self):
        return self._current

     # temperature
    @property
    def temperature(self):
        return self._temperature

    @property
    def color(self):
        return self._color

    @color.setter
    def color(self, new_color):
        new_color = [int(min(max(c, 0), 255)) for c in new_color]
        if len(new_color) > 3 :
            self._color = new_color
            self._push_data('color', [len(new_color)], np.array(new_color, dtype=np.uint8))
        else :
            self._color = new_color
            self._push_value('color', new_color)
    @property
    def time(self):
        return self._time

    @time.setter
    def time(self, new_time):
        self._time = new_time
        self._push_value('time', new_time)

    @property
    def parameters(self):
        return self._parameters

    @parameters.setter
    def parameters(self, new_val):
        self._parameters = new_val
        self._push_value('parameters', new_val)

    def reinit(self):
        self._push_value('reinit', None)

    @property
    def pid(self):
        return self._pid

    @pid.setter
    def pid(self, new_pid):
        self._pid = new_pid
        self._push_value('pid', new_pid)

    @property
    def power_ratio(self):
        self._power_ratio

    @power_ratio.setter
    def power_ratio(self, s):
        s = min(max(s, -100.0), 100.0)
        self._power_ratio = s
        self._push_value("power_ratio",s)

    @property
    def lux(self):
        """ Light in lux. """
        return self._lux

    @property
    def load(self):
        """ force """
        return self._load

    @property
    def volt(self):
        """ Voltage in volt. """
        return self._volt

    @volt.setter
    def volt(self, new_val):
        self._volt = new_val
        self._push_value('volt', new_val)



import os
import time

from ..io.serial_io import Serial
import serial

def serial_ports():
    return Serial.available_hosts()

def serial_discover():
    serial_hosts = serial_ports()
    available_serial = []
    print("Searching for a gate available")
    for serial_host in serial_hosts:
        print("Testing " + str(serial_host))
        try:
            port = serial.Serial(serial_host, 1000000, timeout=0.05)
        except:
            continue
        port.write("{\"discover\": {}}\r".encode())
        port.flush()
        for x in range(10):
            state = port.readline()
            if ('gate'.encode() in state):
                available_serial.append(serial_host)
                continue

        port.close()
    return available_serial

def main():
    gate_list = serial_discover()

    if gate_list:
        print("Available Luos gate on port : " + str(gate_list))
    else:
        print("No gate detected")


if __name__ == '__main__':
    main()



import os

from subprocess import Popen

from ..io.serial_io import Serial


def discover():
    return Serial.available_hosts()


def redirect_to_ws(serial_port, ws_port):
    base_path = os.path.dirname(__file__)

    return Popen(['python', os.path.join(base_path, 'usb2ws.py'),
                  '--serial-port', serial_port,
                  '--ws-port', str(ws_port)])


def main():
    import time
    import argparse

    parser = argparse.ArgumentParser()
    parser.add_argument('cmd', choices=('discover', ))
    args = parser.parse_args()

    if args.cmd == 'discover':
        while True:
            print(discover())
            time.sleep(1.0)


if __name__ == '__main__':
    main()



from .service import Service


class Angle(Service):
    possible_events = {'changed', 'filter_changed'}
    threshold = 10

    def __init__(self, id, alias, device):
        Service.__init__(self, 'Angle', id, alias, device)
        self._value = 0

    @property
    def rot_position(self):
        """ Position in degrees. """
        return self._value

    @rot_position.setter
    def rot_position(self, new_val):
        self._value = new_val
        self._push_value('target_rot_position', new_val)

    def _update(self, new_state):
        Service._update(self, new_state)
        if 'rot_position' in new_state:
            new_val = new_state['rot_position']
            if new_val != self._value:
                self._pub_event('changed', self._value, new_val)

                if abs(new_val - self._value) > self.threshold:
                    self._pub_event('filter_changed',
                                    self._value, new_val)

                self._value = new_val



from .service import Service


class State(Service):
    possible_events = {'changed', 'pressed', 'released'}

    def __init__(self, id, alias, device):
        Service.__init__(self, 'State', id, alias, device)
        self._value = False

    @property
    def state(self):
        return self._value == True

    @state.setter
    def state(self, new_val):
        self._value = new_val
        self._push_value('io_state', new_val)

    def _update(self, new_state):
        Service._update(self, new_state)
        if 'io_state' in new_state:
            new_state = new_state['io_state']
            if new_state != self._value:
                self._pub_event('changed', self._value, new_state)

                evt = 'pressed' if new_state == True else 'released'
                self._pub_event(evt, self._value, new_state)

                self._value = new_state

    def control(self):
        def switch(state):
            self.state = state

        return interact(switch, state=self._value)



from .state import State
from .color import Color
from .motor import Motor
from .servoMotor import ServoMotor
from .angle import Angle
from .distance import Distance
from .gate import Gate
from .imu import Imu
from .light import Light
from .void import Void
from .load import Load
from .voltage import Voltage
from .pipe import Pipe
from .unknown import Unknown


__all__ = [
    'name2mod',
    'State',
    'Color',
    'Motor',
    'ServoMotor',
    'Angle',
    'Distance',
    'Gate',
    'Imu' ,
    'Light',
    'Void',
    'Load',
    'Voltage',
    'Pipe',
    'Unknown'
]

name2mod = {
    'State': State,
    'Color': Color,
    'Motor': Motor,
    'ServoMotor': ServoMotor,
    'Angle': Angle,
    'Distance': Distance,
    'Gate': Gate,
    'Imu': Imu,
    'Light' : Light,
    'Void' : Void,
    'Load' : Load,
    'Voltage' : Voltage,
    'Pipe' : Pipe,
    'Unknown' : Unknown
}



from .service import Service
import time

class Void(Service):

    def __init__(self, id, alias, device):
        Service.__init__(self, 'Void', id, alias, device)
        self._baudrate = 1000000

    def _update(self, new_state):
        Service._update(self, new_state)

    def dxl_detect(self):
        self._push_value('reinit', 0)
        print ("To get new detected Dxl motors usable on pyluos you should recreate your Pyluos object.")

    @property
    def baudrate(self):
        return self._baudrate


    def _set_baudrate(self, baud):
        new_val = [4, baud]
        self._push_value('register', new_val)
        self._baudrate = baud

    @baudrate.setter
    def baudrate(self, baud):
        values = [9600, 19200, 57600, 115200, 200000, 250000, 400000, 500000, 1000000]
        if baud in values :
            self._set_baudrate(baud)
            print ("If you try to recover a motor you should start 'dxl_detect()' command and recreate your Pyluos object.")
        else :
            err = "Possible values are :\n"
            for val in values :
                err = err + "\t- " + str(val) + "\n"
            raise ValueError(err)

    def _factory_reset(self):
        new_val = [0xFF, 0]
        self._push_value('register', new_val)

    def factory_reset(self):
        self._factory_reset()
        print("Motor reseted => baudrate : 1000000, ID : same")
        print("you should start 'dxl_detect()' command and recreate your Pyluos object.")

    def retrieve_dxl(self):
        values = [9600, 19200, 57600, 115200, 200000, 250000, 400000, 500000, 1000000]
        for baud in values :
            self._set_baudrate(baud)
            time.sleep(0.1)
            #self._factory_reset()
            #time.sleep(0.1)
        #self._set_baudrate(57600)
        #time.sleep(0.5)
        self._set_baudrate(1000000)
        time.sleep(0.5)
        self.dxl_detect()
        print("Motor reseted => baudrate : 1000000, ID : same")
        print("recreate your Pyluos object.")



from __future__ import division

from .service import Service, interact


class Motor(Service):
    def __init__(self, id, alias, device):
        Service.__init__(self, 'Motor', id, alias, device)

    @property
    def power_ratio(self):
        self._value

    @power_ratio.setter
    def power_ratio(self, s):
        s = min(max(s, -100.0), 100.0)
        self._value = s
        self._push_value("power_ratio",s)

    def _update(self, new_state):
        Service._update(self, new_state)

    def control(self):
        def move(power_ratio):
            self.power_ratio = power_ratio

        return interact(move, power_ratio=(-100.0, 100.0, 1.0))



from .service import Service, interact
import collections
from copy import copy
import time

compare = lambda x, y: collections.Counter(x) == collections.Counter(y)


class Imu(Service):
    _ACCELL = 9
    _GYRO = 8
    _QUAT = 7
    _COMPASS = 6
    _EULER = 5
    _ROT_MAT = 4
    _PEDO = 3
    _LINEAR_ACCEL = 2
    _GRAVITY_VECTOR = 1
    _HEADING = 0


    def __init__(self, id, alias, device):
        Service.__init__(self, 'Imu', id, alias, device)
        self._config = [False] * (Imu._ACCELL + 1)
        self._config[Imu._QUAT] = True # by default enable quaternion
        self._quaternion = (0, 0, 0, 0)
        self._acceleration = (0, 0, 0)
        self._gyro = (0, 0, 0)
        self._compass = (0, 0, 0)
        self._euler = (0, 0, 0)
        self._rotational_matrix = (0, 0, 0, 0, 0, 0, 0, 0, 0)
        self._pedometer = 0
        self._walk_time = 0
        self._linear_acceleration = (0, 0, 0)
        self._gravity_vector = (0, 0, 0)
        self._heading = 0

    def _convert_config(self):
        return int(''.join(['1' if c else '0' for c in self._config]), 2) # Table read reversly


    def bit(self, i, enable):
        self._config = self._config[:i] + () + self._config[i + 1:]

    @property
    def quaternion(self):
        self.quaternion = True
        return self._quaternion

    @quaternion.setter
    def quaternion(self, enable):
        bak = copy(self._config)
        self._config[Imu._QUAT] = True if enable != 0  else False
        if bak != self._config:
            self._push_value('parameters', self._convert_config())
            time.sleep(0.01)

    @property
    def acceleration(self):
        self.acceleration = True
        return self._acceleration

    @acceleration.setter
    def acceleration(self, enable):
        bak = copy(self._config)
        self._config[Imu._ACCELL] = True if enable != 0  else False
        if bak != self._config:
            self._push_value('parameters', self._convert_config())
            time.sleep(0.01)

    @property
    def gyro(self):
        self.gyro = True
        return self._gyro

    @gyro.setter
    def gyro(self, enable):
        bak = copy(self._config)
        self._config[Imu._GYRO] = True if enable != 0  else False
        if bak != self._config:
            self._push_value('parameters', self._convert_config())
            time.sleep(0.01)

    @property
    def compass(self):
        self.compass = True
        return self._compass

    @compass.setter
    def compass(self, enable):
        bak = copy(self._config)
        self._config[Imu._COMPASS] = True if enable != 0  else False
        if bak != self._config:
            self._push_value('parameters', self._convert_config())
            time.sleep(0.01)

    @property
    def euler(self):
        self.euler = True
        return self._euler

    @euler.setter
    def euler(self, enable):
        bak = copy(self._config)
        self._config[Imu._EULER] = True if enable != 0  else False
        if bak != self._config:
            self._push_value('parameters', self._convert_config())
            time.sleep(0.01)

    @property
    def rotational_matrix(self):
        self.rotational_matrix = True
        return self._rotational_matrix

    @rotational_matrix.setter
    def rotational_matrix(self, enable):
        bak = copy(self._config)
        self._config[Imu._ROT_MAT] = True if enable != 0  else False
        if bak != self._config:
            self._push_value('parameters', self._convert_config())
            time.sleep(0.01)

    @property
    def pedometer(self):
        self.pedometer = True
        return self._pedometer

    @pedometer.setter
    def pedometer(self, enable):
        bak = copy(self._config)
        self._config[Imu._PEDO] = True if enable != 0  else False
        if bak != self._config:
            self._push_value('parameters', self._convert_config())
            time.sleep(0.01)

    @property
    def walk_time(self):
        self.walk_time = True
        return self._walk_time

    @walk_time.setter
    def walk_time(self, enable):
        self.pedometer = enable

    @property
    def linear_acceleration(self):
        self.linear_acceleration = True
        return self._linear_acceleration

    @linear_acceleration.setter
    def linear_acceleration(self, enable):
        bak = copy(self._config)
        self._config[Imu._LINEAR_ACCEL] = True if enable != 0  else False
        if bak != self._config:
            self._push_value('parameters', self._convert_config())
            time.sleep(0.01)

    @property
    def gravity_vector(self):
        self.gravity_vector = True
        return self._gravity_vector

    @gravity_vector.setter
    def gravity_vector(self, enable):
        bak = copy(self._config)
        self._config[Imu._GRAVITY_VECTOR] = True if enable != 0  else False
        if bak != self._config:
            self._push_value('parameters', self._convert_config())
            time.sleep(0.01)

    @property
    def heading(self):
        self.heading = True
        return self._heading

    @heading.setter
    def heading(self, enable):
        bak = copy(self._config)
        self._config[Imu._HEADING] = True if enable != 0  else False
        if bak != self._config:
            self._push_value('parameters', self._convert_config())
            time.sleep(0.01)


    def _update(self, new_state):
        Service._update(self, new_state)
        if 'quaternion' in new_state:
            self._quaternion = new_state['quaternion']
        if 'accel' in new_state:
            self._acceleration = new_state['accel']
        if 'gyro' in new_state:
            self._gyro = new_state['gyro']
        if 'compass' in new_state:
            self._compass = new_state['compass']
        if 'euler' in new_state:
            self._euler = new_state['euler']
        if 'rotational_matrix' in new_state:
            self._rotational_matrix = new_state['rotational_matrix']
        if 'pedometer' in new_state:
            self._pedometer = new_state['pedometer']
        if 'walk_time' in new_state:
            self._walk_time = new_state['walk_time']
        if 'linear_accel' in new_state:
            self._linear_acceleration = new_state['linear_accel']
        if 'gravity_vector' in new_state:
            self._gravity_vector = new_state['gravity_vector']
        if 'heading' in new_state:
            self._heading = new_state['heading']

    def control(self):
        def change_config(accel, gyro, quat, compass, euler, rot_mat, pedo, linear_accel, gravity_vector, heading):
            self.acceleration = accel
            self.gyro = gyro
            self.quaternion = quat
            self.compass = compass
            self.euler = euler
            self.rotational_matrix = rot_mat
            self.pedometer = pedo
            self.linear_acceleration = linear_accel
            self.gravity_vector = gravity_vector
            self.heading = heading
            self._push_value('parameters', self._convert_config())

        return interact(change_config,
                        accel=self._config[Imu._ACCELL],
                        gyro=self._config[Imu._GYRO],
                        quat=self._config[Imu._QUAT],
                        compass=self._config[Imu._COMPASS],
                        euler=self._config[Imu._EULER],
                        rot_mat=self._config[Imu._ROT_MAT],
                        pedo=self._config[Imu._PEDO],
                        linear_accel=self._config[Imu._LINEAR_ACCEL],
                        gravity_vector=self._config[Imu._GRAVITY_VECTOR],
                        heading=self._config[Imu._HEADING])



from .service import Service


class Load(Service):
    possible_events = {'changed', 'filter_changed'}
    threshold = 10

    def __init__(self, id, alias, device):
        Service.__init__(self, 'Load', id, alias, device)
        self._load = 0.0
        self._offset = 0.0
        self._scale = 1.0

    @property
    def load(self):
        """ force """
        return self._load

    def tare(self):
        # measure and offset
        self._push_value("reinit", None)
        time.sleep(1.0)

    @property
    def offset(self):
            return self._offset

    @offset.setter
    def offset(self, value):
            self._offset = value
            self._push_value("offset",value)

    @property
    def scale(self):
            return self._scale

    @scale.setter
    def scale(self, value):
            self._scale = value
            self._push_value("resolution",value)

    def _update(self, new_state):
        Service._update(self, new_state)
        if 'force' in new_state:
            new_force = new_state['force']
            if new_force != self._value:
                self._pub_event('changed', self._value, new_force)
                if abs(new_force - self._load) > self.threshold:
                    self._pub_event('filter_changed',
                                    self._value, new_force)

                self._load = new_force

    def control(self):
        def change_config(offset, scale):
            # report config
            self.offset = offset
            self.scale = scale

        w = interact(change_config,
                    offset = self.offset,
                    scale = self.scale)



from __future__ import division
from __future__ import division
import collections
from copy import copy
import time

from .service import Service, interact
import numpy as np


class ServoMotor(Service):
    # target modes
    _MODE_COMPLIANT = 13
    _MODE_POWER = 12
    _MODE_TORQUE = 11
    _MODE_ANGULAR_SPEED = 10
    _MODE_ANGULAR_POSITION = 9
    _MODE_LINEAR_SPEED = 8
    _MODE_LINEAR_POSITION = 7
    # report modes
    _ANGULAR_POSITION = 6
    _ANGULAR_SPEED = 5
    _LINEAR_POSITION = 4
    _LINEAR_SPEED = 3
    _CURRENT = 2
    _TEMPERATURE = 1
    _TORQUE = 0

    # control modes
    _PLAY = 0
    _PAUSE = 1
    _STOP = 2
    _REC = 4

    def __init__(self, id, alias, device):
        Service.__init__(self, 'ServoMotor', id, alias, device)
        self._config = [False] * (ServoMotor._MODE_COMPLIANT + 1)
        # default configs, enable compliant, power_mode, and rotation position report
        self._config[ServoMotor._MODE_COMPLIANT] = True
        self._config[ServoMotor._MODE_POWER] = True
        self._config[ServoMotor._ANGULAR_POSITION] = True

        #configuration
        self._positionPid = [0.0, 0.0, 0.0]
        self._speedPid = [0.0, 0.0, 0.0]
        self._resolution = 16 # encoder resolution
        self._reduction = 131 # mechanical reduction after encoder
        self._dimension = 100 # Wheel size (mm)
        self._limit_rot_position = None
        self._limit_trans_position = None
        self._limit_rot_speed = None
        self._limit_trans_speed = None
        self._limit_power = 100.0
        self._limit_current = 6.0
        self._sampling_freq = 100.0
        self._control = 0

        #targets
        self._compliant = True
        self._target_power = 0.0
        self._target_rot_speed = 0.0
        self._target_rot_position = 0.0
        self._target_trans_speed = 0.0
        self._target_trans_position = 0.0

        # report modes
        self._rot_position = 0.0
        self._rot_speed = 0.0
        self._trans_position= 0.0
        self._trans_speed = 0.0
        self._current = 0.0
        self._temperature = 0.0

    def _convert_config(self):
        return int(''.join(['1' if c else '0' for c in self._config]), 2) # Table read reversly

    def bit(self, i, enable):
        self._config = self._config[:i] + () + self._config[i + 1:]

#************************** configurations *****************************

    def play(self):
        if (self._control >= self._REC):
            self._control = self._PLAY + self._REC
        else :
            self._control = self._PLAY
        self._push_value('control', self._control)

    def pause(self):
        if (self._control >= self._REC):
            self._control = self._PAUSE + self._REC
        else :
            self._control = self._PAUSE
        self._push_value('control', self._control)

    def stop(self):
        # also stop recording
        self._control = self._STOP
        self._push_value('control', self._control)

    def rec(self, enable):
        if (self._control >= self._REC):
            if (enable == False):
                self._control = self._control - self._REC
        else :
            if (enable == True):
                self._control = self._control + self._REC
        self._push_value('control', self._control)

    def setToZero(self):
        self._push_value('reinit', None)

    @property
    def sampling_freq(self):
        return self._sampling_freq

    @sampling_freq.setter
    def sampling_freq(self, sampling_freq):
        self._sampling_freq = sampling_freq
        self._push_value("time", 1.0 / sampling_freq)

    @property
    def positionPid(self):
        return self._positionPid

    @positionPid.setter
    def positionPid(self, new_pid):
        bak = copy(self._config)
        self.compliant = True
        self.rot_position_mode = True
        self.rot_speed_mode = False
        time.sleep(0.2)
        self._positionPid = new_pid
        self._push_value('pid', new_pid)
        time.sleep(0.2)
        self._config = bak
        self._push_value('parameters', self._convert_config())
        time.sleep(0.01)

    @property
    def speedPid(self):
        return self._speedPid

    @speedPid.setter
    def speedPid(self, new_pid):
        bak = copy(self._config)
        self.compliant = True
        self.rot_position_mode = False
        self.rot_speed_mode = True
        time.sleep(0.2)
        self._speedPid = new_pid
        self._push_value('pid', new_pid)
        time.sleep(0.2)
        self._config = bak
        self._push_value('parameters', self._convert_config())
        time.sleep(0.01)

    @property
    def encoder_res(self):
        return self._resolution

    @encoder_res.setter
    def encoder_res(self, s):
        self._resolution = s
        self._push_value("resolution", s)

    @property
    def reduction(self):
        return self._resolution

    @reduction.setter
    def reduction(self, s):
        self._reduction = s
        self._push_value("reduction", s)

    @property
    def wheel_size(self):
        return self._dimension

    @wheel_size.setter
    def wheel_size(self, s):
        self._dimension = s
        self._push_value("dimension", s)

    @property
    def limit_rot_position(self):
        return self._limit_rot_position

    @limit_rot_position.setter
    def limit_rot_position(self, s):
        self._limit_rot_position = s
        self._push_value("limit_rot_position", s)

    @property
    def limit_trans_position(self):
        return self._limit_trans_position

    @limit_trans_position.setter
    def limit_trans_position(self, s):
        self._limit_trans_position = s
        self._push_value("limit_trans_position", s)

    @property
    def limit_rot_speed(self):
        return self._limit_rot_speed

    @limit_rot_speed.setter
    def limit_rot_speed(self, s):
        self._limit_rot_speed = s
        self._push_value("limit_rot_speed", s)

    @property
    def limit_trans_speed(self):
        return self._limit_trans_speed

    @limit_trans_speed.setter
    def limit_trans_speed(self, s):
        self._limit_trans_speed = s
        self._push_value("limit_trans_speed", s)

    @property
    def limit_power(self):
        return self._limit_power

    @limit_power.setter
    def limit_power(self, s):
        self._limit_power = abs(s)
        s = min(s, 100.0)
        self._push_value("limit_power", s)

    @property
    def limit_current(self):
        return self._limit_current

    @limit_current.setter
    def limit_current(self, s):
        self._limit_current = s
        self._push_value("limit_current", s)

#************************** target modes *****************************

    # compliant
    @property
    def compliant(self):
        return self._config[ServoMotor._MODE_COMPLIANT]

    @compliant.setter
    def compliant(self, enable):
        self._config[ServoMotor._MODE_COMPLIANT] = True if enable != 0  else False
        self._compliant = enable
        self._push_value('parameters', self._convert_config())
        if (enable == False):
            self._target_rot_position = self._rot_position
        time.sleep(0.01)

    # power
    @property
    def power_ratio(self):
        if (self._config[ServoMotor._MODE_POWER] != True):
            print("power mode is not enabled in the service please use 'device.service.power_mode = True' to enable it")
            return
        return self._target_power

    @power_ratio.setter
    def power_ratio(self, s):
        if (self._config[ServoMotor._MODE_POWER] != True):
            print("power mode is not enabled in the service please use 'device.service.power_mode = True' to enable it")
        s = min(max(s, -100.0), 100.0)
        #if s != self._target_power:
        self._target_power = s
        self._push_value("power_ratio",s)

    @property
    def power_mode(self):
        return self._config[ServoMotor._MODE_POWER]

    @power_mode.setter
    def power_mode(self, enable):
        self._config[ServoMotor._MODE_POWER] = True if enable != 0  else False
        if (enable == True) :
            self._config[ServoMotor._MODE_ANGULAR_SPEED] = False
            self._config[ServoMotor._MODE_ANGULAR_POSITION] = False
            self._config[ServoMotor._MODE_LINEAR_SPEED] = False
            self._config[ServoMotor._MODE_LINEAR_POSITION] = False
        self._push_value('parameters', self._convert_config())
        time.sleep(0.01)

    # rotation speed
    @property
    def target_rot_speed(self):
        if (self._config[ServoMotor._MODE_ANGULAR_SPEED] != True):
            print("rotation speed mode could be not enabled in the service please use 'device.service.rot_speed_mode = True' to enable it")
        return self._target_rot_speed

    @target_rot_speed.setter
    def target_rot_speed(self, s):
        if (self._config[ServoMotor._MODE_ANGULAR_SPEED] != True):
            print("rotation speed mode could be not enabled in the service please use 'device.service.rot_speed_mode = True' to enable it")
        self._target_rot_speed = s
        self._push_value("target_rot_speed", s)

    @property
    def rot_speed_mode(self):
        return self._config[ServoMotor._MODE_ANGULAR_SPEED]

    @rot_speed_mode.setter
    def rot_speed_mode(self, enable):
        self._config[ServoMotor._MODE_ANGULAR_SPEED] = True if enable != 0  else False
        if (enable == True) :
            self._config[ServoMotor._MODE_LINEAR_SPEED] = False
            self._config[ServoMotor._MODE_POWER] = False
        self._push_value('parameters', self._convert_config())
        time.sleep(0.01)

    # rotation position
    @property
    def target_rot_position(self):
        if (self._config[ServoMotor._MODE_ANGULAR_POSITION] != True):
            print("rotation position mode could be not enabled in the service please use 'device.service.rot_position_mode = True' to enable it")
        return self._target_rot_position

    @target_rot_position.setter
    def target_rot_position(self, s):
        if (self._config[ServoMotor._MODE_ANGULAR_POSITION] != True):
            print("rotation position mode could be not enabled in the service please use 'device.service.rot_position_mode = True' to enable it")
        self._target_rot_position = s
        if hasattr(s, "__len__"):
            self._push_data('target_rot_position', [len(s) * 4], np.array(s, dtype=np.float32)) # multiplying by the size of float32
        else :
            self._push_value("target_rot_position", s)

    @property
    def rot_position_mode(self):
        return self._config[ServoMotor._MODE_ANGULAR_POSITION]

    @rot_position_mode.setter
    def rot_position_mode(self, enable):
        self._config[ServoMotor._MODE_ANGULAR_POSITION] = True if enable != 0  else False
        if (enable == True) :
            self._config[ServoMotor._MODE_LINEAR_POSITION] = False
            self._config[ServoMotor._MODE_POWER] = False
        self._push_value('parameters', self._convert_config())
        time.sleep(0.01)

    # translation speed
    @property
    def target_trans_speed(self):
        if (self._config[ServoMotor._MODE_LINEAR_SPEED] != True):
            print("translation speed mode could be not enabled in the service please use 'device.service.trans_speed_mode = True' to enable it")
        return self._target_trans_speed

    @target_trans_speed.setter
    def target_trans_speed(self, s):
        if (self._config[ServoMotor._MODE_LINEAR_SPEED] != True):
            print("translation speed mode could be not enabled in the service please use 'device.service.trans_speed_mode = True' to enable it")
        self._target_trans_speed = s
        self._push_value("target_trans_speed", s)

    @property
    def trans_speed_mode(self):
        return self._config[ServoMotor._MODE_LINEAR_SPEED]

    @trans_speed_mode.setter
    def trans_speed_mode(self, enable):
        self._config[ServoMotor._MODE_LINEAR_SPEED] = True if enable != 0  else False
        if (enable == True) :
            self._config[ServoMotor._MODE_ANGULAR_SPEED] = False
            self._config[ServoMotor._MODE_POWER] = False
        self._push_value('parameters', self._convert_config())
        time.sleep(0.01)

    # translation position
    @property
    def target_trans_position(self):
        if (self._config[ServoMotor._MODE_LINEAR_POSITION] != True):
            print("translation speed mode could be not enabled in the service please use 'device.service.trans_pos_mode = True' to enable it")
        return self._target_trans_position

    @target_trans_position.setter
    def target_trans_position(self, s):
        if (self._config[ServoMotor._MODE_LINEAR_POSITION] != True):
            print("translation speed mode could be not enabled in the service please use 'device.service.trans_position_mode = True' to enable it")
        self._target_trans_position = s
        if hasattr(s, "__len__"):
            self._push_value('target_trans_position', [len(s) * 4]) # multiplying by the size of float32
            self._push_data(np.array(s, dtype=np.float32))
        else :
            self._push_value("target_trans_position", s)

    @property
    def trans_position_mode(self):
        return self._config[ServoMotor._MODE_LINEAR_POSITION]

    @trans_position_mode.setter
    def trans_position_mode(self, enable):
        self._config[ServoMotor._MODE_LINEAR_POSITION] = True if enable != 0  else False
        if (enable == True) :
            self._config[ServoMotor._MODE_ANGULAR_POSITION] = False
            self._config[ServoMotor._MODE_POWER] = False
        self._push_value('parameters', self._convert_config())
        time.sleep(0.01)
#************************** report modes *****************************

    # rotation position
    @property
    def rot_position(self):
        if (self._config[ServoMotor._ANGULAR_POSITION] != True):
            self.rot_position = True
        return self._rot_position

    @rot_position.setter
    def rot_position(self, enable):
        self._config[ServoMotor._ANGULAR_POSITION] = True if enable != 0  else False
        self._push_value('parameters', self._convert_config())
        time.sleep(0.01)

    # rotation speed
    @property
    def rot_speed(self):
        if (self._config[ServoMotor._ANGULAR_SPEED] != True):
            self.rot_speed = True
        return self._rot_speed

    @rot_speed.setter
    def rot_speed(self, enable):
        self._config[ServoMotor._ANGULAR_SPEED] = True if enable != 0  else False
        self._push_value('parameters', self._convert_config())
        time.sleep(0.01)

    # translation position
    @property
    def trans_position(self):
        if (self._config[ServoMotor._LINEAR_POSITION] != True):
            self.trans_position = True
        return self._rot_position

    @trans_position.setter
    def trans_position(self, enable):
        self._config[ServoMotor._LINEAR_POSITION] = True if enable != 0  else False
        self._push_value('parameters', self._convert_config())
        time.sleep(0.01)

    # translation speed
    @property
    def trans_speed(self):
        if (self._config[ServoMotor._LINEAR_SPEED] != True):
            self.trans_speed = True
        return self._rot_speed

    @trans_speed.setter
    def trans_speed(self, enable):
        self._config[ServoMotor._LINEAR_SPEED] = True if enable != 0  else False
        self._push_value('parameters', self._convert_config())
        time.sleep(0.01)

    # current
    @property
    def current(self):
        if (self._config[ServoMotor._CURRENT] != True):
            self.current = True
        return self._current

    @current.setter
    def current(self, enable):
        self._config[ServoMotor._CURRENT] = True if enable != 0  else False
        self._push_value('parameters', self._convert_config())
        time.sleep(0.01)

    # temperature
    @property
    def temperature(self):
        if (self._config[ServoMotor._TEMPERATURE] != True):
            self.temperature = True
        return self._temperature

    @temperature.setter
    def temperature(self, enable):
        self._config[ServoMotor._TEMPERATURE] = True if enable != 0  else False
        self._push_value('parameters', self._convert_config())
        time.sleep(0.01)

#************************** custom motor type commands *****************************

    def dxl_set_id(self, id):
        self._push_value('set_id', id)

    def dxl_detect(self):
        self._push_value('reinit', 0)
        print ("To get new detected Dxl motors usable on pyluos you should recreate your Pyluos object.")

    def dxl_register(self, register, val):
        new_val = [register, val]
        self._push_value('register', new_val)

#************************** controls and updates *****************************

    def _update(self, new_state):
        Service._update(self, new_state)
        if 'rot_position' in new_state:
            self._rot_position = new_state['rot_position']
        if 'rot_speed' in new_state:
            self._rot_speed = new_state['rot_speed']
        if 'trans_position' in new_state:
            self._trans_position = new_state['trans_position']
        if 'trans_speed' in new_state:
            self._trans_speed = new_state['trans_speed']
        if 'current' in new_state:
            self._current = new_state['current']
        if 'temperature' in new_state:
            self._temperature = new_state['temperature']

    def control(self):
        def change_config(rot_speed_report, rot_position_report, trans_speed_report, trans_position_report, current_report, compliant_mode, power_mode, power_ratio, rot_speed_mode, rot_speed, rot_position_mode, rot_position, trans_speed_mode, trans_speed, trans_position_mode, trans_position):
            # report config
            self.rot_speed = rot_speed_report
            self.rot_position = rot_position_report
            self.trans_speed = trans_speed_report
            self.trans_position = trans_position_report
            self.current = current_report
            # target mode
            self.compliant = compliant_mode
            self.power_mode = power_mode
            if (power_mode) :
                self.power_ratio = power_ratio

            self.rot_speed_mode = rot_speed_mode
            if (rot_speed_mode) :
                self.target_rot_speed = rot_speed

            self.rot_position_mode = rot_position_mode
            if (rot_position_mode) :
                self.target_rot_position = rot_position

            self.trans_speed_mode = trans_speed_mode
            if (trans_speed_mode) :
                self.target_trans_speed = trans_speed

            self.trans_position_mode = trans_position_mode
            if (trans_position_mode) :
                self.target_trans_position = trans_position

        w = interact(change_config,
                        rot_speed_report = self._config[ServoMotor._ANGULAR_SPEED],
                        rot_position_report = self._config[ServoMotor._ANGULAR_POSITION],
                        trans_speed_report = self._config[ServoMotor._LINEAR_SPEED],
                        trans_position_report = self._config[ServoMotor._LINEAR_POSITION],
                        current_report = self._config[ServoMotor._CURRENT],
                        temperature_report = self._config[ServoMotor._TEMPERATURE],

                        compliant_mode = self._config[ServoMotor._MODE_COMPLIANT],
                        power_mode = self._config[ServoMotor._MODE_POWER],
                        power_ratio=(-100.0, 100.0, 1.0),
                        rot_speed_mode = self._config[ServoMotor._MODE_ANGULAR_SPEED],
                        rot_speed = (-300.0, 300.0, 1.0),
                        rot_position_mode = self._config[ServoMotor._MODE_ANGULAR_POSITION],
                        rot_position = (-360.0, 360.0, 1.0),
                        trans_speed_mode = self._config[ServoMotor._MODE_LINEAR_SPEED],
                        trans_speed = (-1000.0, 1000.0, 1.0),
                        trans_position_mode = self._config[ServoMotor._MODE_LINEAR_POSITION],
                        trans_position = (-1000.0, 1000.0, 1.0))



# -*- coding: utf-8 -*-
import os
import sys
import json
import time
import logging
import threading
import logging.config
import numpy as np

from datetime import datetime
from collections import defaultdict

from .io import discover_hosts, io_from_host, Ws
from .services import name2mod

from anytree import AnyNode, RenderTree, DoubleStyle


def run_from_unittest():
    return 'unittest' in sys.services


known_host = {
    'ergo': ['/dev/cu.usbserial-DN2AAOVK', '/dev/cu.usbserial-DN2YEFLN'],
    'handy': ['/dev/cu.usbserial-DN2X236E'],
    'eddy': ['pi-gate.local'],
}


class contList(list):
    def __repr__(self):
        s = '-------------------------------------------------\n'
        s += '{:<20s}{:<20s}{:<5s}\n'.format("Type", "Alias", "ID")
        s += '-------------------------------------------------\n'
        for elem in self:
            s += '{:<20s}{:<20s}{:<5d}\n'.format(elem.type, elem.alias, elem.id)
        return s

class nodeList(list):
    def __repr__(self):
        # Display the topology
        s = ''
        prefill = ''
        prechild = False
        for pre, fill, node in RenderTree(self[0], style=DoubleStyle()):
            child = []
            if (node.parent == None):
                branch = "  ┃  "
                for i,x in enumerate(node.port_table):
                    child.append(i)
            else:
                l_port_id = '?'
                for i,x in enumerate(node.parent.port_table):
                    if (x == node.id):
                        l_port_id = str(i)
                r_port_id = node.port_table.index(min(node.port_table))
                for i,x in enumerate(node.port_table):
                    if ((i != r_port_id) and (x != 65535)):
                        child.append(i)
                branch = str(l_port_id) + ">┃" + str(r_port_id) + " "
            prefill = (prefill[:len(fill)]) if len(prefill) > len(fill) else prefill
            s +='{:<{fillsize}s}'.format(prefill, fillsize=len(fill))
            if (prechild == True):
                position = -4
                s = s[:position] + '║' + s[position+1:]
            s += "  ┏━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━┓\n"
            tmpstr = "%s╭node %s" % (branch, node.id)
            s += pre + '{:^10s}'.format(tmpstr)
            if (node.certified == True):
                s += '{:^41s}'.format("Certified") + "┃\n"
            else:
                s += '{:^41s}'.format("/!\\ Not certified") + "┃\n"
            s += fill + "  ┃  │  " + '{:<20s}{:<20s}{:<5s}'.format("Type", "Alias", "ID")+ "┃\n"
            for y,elem in enumerate(node.services):
                if (y == (len(node.services)-1)):
                    s += fill + "  ┃  ╰> " + '{:<20s}{:<20s}{:<5d}'.format(elem.type, elem.alias, elem.id)+ "┃\n"
                else:
                    s += fill + "  ┃  ├> " + '{:<20s}{:<20s}{:<5d}'.format(elem.type, elem.alias, elem.id) + "┃\n"
            if (not child):
                s += fill + " >┗━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━┛\n"
                prechild = False
            else:
                s += fill + "╔>┗━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━┛\n"
                prechild = True
            prefill = fill
        return s

class Device(object):
    _heartbeat_timeout = 5  # in sec.
    _max_alias_length = 15
    _base_log_conf = os.path.join(os.path.dirname(__file__),
                                  'logging_conf.json')

    @classmethod
    def discover(cls):
        hosts = discover_hosts()

        possibilities = {
            k: [h for h in v if h in hosts]
            for k, v in known_host.items()
        }

        return possibilities

    def __init__(self, host,
                 IO=None,
                 log_conf=_base_log_conf,
                 test_mode=False,
                 background_task=True,
                 *args, **kwargs):
        if IO is not None:
            self._io = IO(host=host, *args, **kwargs)
        else:
            self._io = io_from_host(host=host,
                                    *args, **kwargs)

        if os.path.exists(log_conf):
            with open(log_conf) as f:
                config = json.load(f)
            logging.config.dictConfig(config)

        self.logger = logging.getLogger(__name__)
        self.logger.info('Connected to "{}".'.format(host))

        self._send_lock = threading.Lock()
        self._cmd_lock = threading.Lock()

        # We force a first poll to setup our model.
        self._setup()
        self.logger.info('Device setup.')

        self._last_update = time.time()
        self._running = True
        self._pause = False

        if(background_task == True):
            # Setup both poll/push synchronization loops.
            self._poll_bg = threading.Thread(target=self._poll_and_up)
            self._poll_bg.daemon = True
            self._poll_bg.start()
        self._baudrate = 1000000

    def close(self):
        self._running = False
        self._poll_bg.join()
        self._io.close()

    @property
    def baudrate(self):
        return self._baudrate

    @baudrate.setter
    def baudrate(self, baudrate):
        self._send({'baudrate': baudrate})
        self._baudrate = baudrate
        time.sleep(0.01)

    def benchmark(self, target_id, data, repetition):
        data = np.array(data, dtype=np.uint8)
        self._bench_settings = {'benchmark': {'target': target_id, 'repetitions': repetition, 'data': [len(data)]}}
        self._bench_Data = data.tobytes()
        self._write( json.dumps(self._bench_settings).encode() + '\r'.encode() + self._bench_Data)

        state = self._poll_once()
        startTime = time.time()
        retry = 0
        while ('benchmark' not in state):
            state = self._poll_once()
            if (time.time()-startTime > 30):
                self._write( json.dumps(self._bench_settings).encode() + '\r'.encode() + self._bench_Data)
                retry = retry+1
                if (retry == 3):
                    return (0, 100)
                startTime = time.time()

        #self._pause = False
        return (state['benchmark']['data_rate'], state['benchmark']['fail_rate'])

    def pause(self):
        self._pause = True
        time.sleep(1)

    def play(self):
        self._pause = False

    def _setup(self):
        self.logger.info('Sending detection signal.')
        self._send({'detection': {}})
        self.logger.info('Waiting for routing table...')
        startTime = time.time()
        state = self._poll_once()
        while ('routing_table' not in state):
            if ('route_table' in state):
                self.logger.info("Watch out the Luos revision you are using on your board is too old to work with this revision on pyluos.\n Please consider updating Luos on your boards")
                return
            state = self._poll_once()
            if (time.time()-startTime > 1):
                self._send({'detection': {}})
                startTime = time.time()
        # Create nodes
        self._services = []
        self._nodes = []
        for i, node in enumerate(state['routing_table']):
            if ('node_id' not in node):
                self.logger.info("Watch out the Luos revision you are using on your board is too old to work with this revision on pyluos.\n Please consider updating Luos on your boards")
            parent_elem = None
            # find a parent and create a link
            if (min(node["port_table"]) < node["services"][0]["id"]):
                parent_id = min(node["port_table"])
                for elem in self._nodes:
                    if (elem.id == parent_id):
                        parent_elem = elem
                        break;
            # create the node
            self._nodes.append(AnyNode(id=node["node_id"], certified=node["certified"], parent=parent_elem, port_table=node["port_table"]))

            filtered_services = contList([mod for mod in node["services"]
                                if 'type' in mod and mod['type'] in name2mod.keys()])
            # Create a list of services in the node
            self._nodes[i].services = [
                name2mod[mod['type']](id=mod['id'],
                                      alias=mod['alias'],
                                      device=self)
                for mod in filtered_services
                if 'type' in mod and 'id' in mod and 'alias' in mod
            ]
            # Create a list of services of the entire device
            self._services = self._services + self._nodes[i].services
            for mod in self._nodes[i].services:
                setattr(self, mod.alias, mod)

        self._cmd = defaultdict(lambda: defaultdict(lambda: None))
        self._cmd_data = []
        self._binary = []

        # We push our current state to make sure that
        # both our model and the hardware are synced.
        self._push_once()

    @property
    def services(self):
        return contList(self._services)

    @property
    def nodes(self):
        return nodeList(self._nodes)

    # Poll state from hardware.
    def _poll_once(self):
        self._state = self._io.read()
        self._state['timestamp'] = time.time()
        return self._state

    def _poll_and_up(self):
        while self._running:
            if not self._pause :
                state = self._poll_once()
                self._update(state)
                self._push_once()
            else :
                time.sleep(0.1)

    # Update our model with the new state.
    def _update(self, new_state):
        if 'dead_service' in new_state :
            #we have lost a service put a flag on this service
            alias = new_state['dead_service']
            if hasattr(self, alias):
                getattr(self, alias)._kill()
        if 'assert' in new_state :
            # A node assert, print assert informations
            if (('node_id' in new_state['assert']) and ('file' in new_state['assert']) and ('line' in new_state['assert'])):
                s = "************************* ASSERT *************************\n"
                s += "*  Node " + str(new_state['assert']['node_id']) + " assert in file " + new_state['assert']['file'] + " line " + str(new_state['assert']['line'])
                s += "\n**********************************************************"
                print (s)
        if 'services' not in new_state:
            return

        for alias, mod in new_state['services'].items():
            if hasattr(self, alias):
                getattr(self, alias)._update(mod)

        self._last_update = time.time()

    def update_cmd(self, alias, key, val):
        with self._cmd_lock:
            self._cmd[alias][key] = val

    def update_data(self, alias, key, val, data):
        with self._cmd_lock:
            self._cmd_data.append({alias: {key: val}})
            self._binary.append(data.tobytes())

    def _push_once(self):
        with self._cmd_lock:
            if self._cmd:
                self._write( json.dumps({'services': self._cmd}).encode())
                self._cmd = defaultdict(lambda: defaultdict(lambda: None))
            for cmd, binary in zip(self._cmd_data, self._binary):
                time.sleep(0.01)
                self._write( json.dumps({'services': cmd}).encode() + '\r'.encode() + binary)

            self._cmd_data = []
            self._binary = []


    def _send(self, msg):
        with self._send_lock:
            self._io.send(msg)

    def _write(self, data):
        with self._send_lock:
            self._io.write(data)



import json
import logging


class IOHandler(object):
    @classmethod
    def is_host_compatible(cls, host):
        return False

    def __init__(self, host):
        raise NotImplementedError

    def is_ready(self):
        raise NotImplementedError

    def read(self, trials=5):
        try:
            data = self.recv()
            return self.loads(data)
        except Exception as e:
            logging.getLogger(__name__).debug('Msg read failed: {}'.format(str(e)))
            if trials == 0:
                raise e

        return self.read(trials - 1)

    def recv(self):
        raise NotImplementedError

    def send(self, msg):
        self.write(self.dumps(msg))

    def write(self, data):
        self.write(data)

    def loads(self, data):
        if type(data) == bytes:
            data = data.decode()
        return json.loads(data)

    def dumps(self, msg):
        return json.dumps(msg).encode()


from .ws import Ws
from .serial_io import Serial

IOs = [Serial, Ws]


def io_from_host(host, *args, **kwargs):
    for cls in IOs:
        if cls.is_host_compatible(host):
            return cls(host=host, *args, **kwargs)

    raise ValueError('No corresponding IO found (among {}).'.format(discover_hosts))


def discover_hosts():
    return sum([io.available_hosts() for io in IOs], [])



from .service import Service


class Voltage(Service):
    possible_events = {'changed', 'filter_changed'}

    def __init__(self, id, alias, device):
        Service.__init__(self, 'Voltage', id, alias, device)
        self._value = 0
        self.threshold = 1.0

    @property
    def volt(self):
        """ Voltage in volt. """
        return self._value

    @volt.setter
    def volt(self, new_val):
        self._value = new_val
        self._push_value('volt', new_val)

    def _update(self, new_state):
        Service._update(self, new_state)
        if 'volt' in new_state:
            new_val = new_state['volt']
            if new_val != self._value:
                self._pub_event('changed', self._value, new_val)
                if abs(new_val - self._value) > self.threshold:
                    self._pub_event('filter_changed',
                                    self._value, new_val)
            self._value = new_val

    def control(self):
        def move(val):
            self._value = val

        return interact(move, val=(0.0, 3.3, 0.1))



from .service import Service


class Gate(Service):

    def __init__(self, id, alias, device):
        Service.__init__(self, 'Gate', id, alias, device)

    def _update(self, new_state):
        Service._update(self, new_state)

    def control(self):
        def delay(delay):
            self._value = delay

        return interact(delay, delay=(0, 100, 1))



import os
import socket
import websocket

import sys
if sys.version_info >= (3, 0):
    import queue
else:
    import Queue as queue

from threading import Event, Thread
from . import IOHandler

def resolve_hostname(hostname, port):
    # We do our own mDNS resolution
    # to enforce we only search for IPV4 address
    # and avoid a 5s timeout in the websocket on the ESP
    # See https://github.com/esp8266/Arduino/issues/2110

    addrinfo = socket.getaddrinfo(hostname, port,
                                  socket.AF_INET, 0,
                                  socket.SOL_TCP)
    addr = addrinfo[0][4][0]
    return addr


class Ws(IOHandler):

    @classmethod
    def is_host_compatible(cls, host):
        try:
            socket.inet_pton(socket.AF_INET, host)
            return True
        except socket.error:
            return host.endswith('.local')

    @classmethod
    def available_hosts(cls):
        hosts = ['pi-gate.local']

        return [
            ip
            for ip in hosts
            if os.system('ping -c 1 -W1 -t1 {} > /dev/null 2>&1'.format(ip)) == 0
        ]

    def __init__(self, host, port=9342):
        host = resolve_hostname(host, port)

        self._ws = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self._ws.connect((host, port))

        self._msg = queue.Queue(100)
        self._running = True

        self._poll_loop = Thread(target=self._poll)
        self._poll_loop.daemon = True
        self._poll_loop.start()

    def is_ready(self):
        return True

    def recv(self):
        return self._msg.get()

    def write(self, data):
        self._ws.send(data + '\r'.encode())

    def close(self):
        self._running = False
        self._poll_loop.join()
        self._ws.close()

    def _poll(self):
        def extract_line(s):
            j = s.find(b'\n')
            if j == -1:
                return b'', s
            # Sometimes the begin of serial data can be wrong remove it
            # Find the first '{'

            x = s.find(b'{')
            if x == -1:
                return b'', s[j + 1:]

            return s[x:j], s[j + 1:]

        buff = b''

        while self._running:
            s = self._ws.recv(4096)

            buff = buff + s
            while self._running:
                line, buff = extract_line(buff)
                if not len(line):
                    break
                if self._msg.full():
                    self._msg.get()
                self._msg.put(line)



#!/usr/bin/env python

import imp

from setuptools import setup, find_packages

version = imp.load_source('pyluos.version', 'pyluos/version.py')

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(name='pyluos',
      version=version.version,
      author="Luos",
      author_email="hello@luos.io",
      url="https://docs.luos.io/pages/high/pyluos.html",
      description="Python library to set the high level behavior of your device based on Luos embedded system.",
      license='MIT',
      packages=find_packages(),
      install_requires=['future',
                        'websocket-client',
                        'pyserial>3',
                        'SimpleWebSocketServer',
                        'zeroconf',
                        'numpy',
                        'anytree',
                        'crc8'
                        ],
      extras_require={
          'tests': ['pytest', 'flake8'],
          'jupyter-integration': ['ipywidgets'],
      },
      entry_points={
          'console_scripts': [
              'pyluos-wifi-gate = pyluos.tools.wifi_gate:main',
              'pyluos-usb-gate = pyluos.tools.usb_gate:main',
              'pyluos-usb2ws = pyluos.tools.usb2ws:main',
              'pyluos-bootloader = pyluos.tools.bootloader:main',
              'pyluos-shell = pyluos.tools.shell:main',
              'pyluos-discover = pyluos.tools.discover:main'
          ],
      },
      )



from collections import defaultdict, namedtuple

import logging
import time

try:
    from ipywidgets import interact
    from ipywidgets import widgets
except ImportError:
    def interact(*args, **kwargs):
        msg = 'You first have to install ipywidgets to use the control widgets.'
        logging.getLogger(__name__).warning(msg)
        return None

    def widgets(*args, **kwargs):
        msg = 'You first have to install ipywidgets to use the control widgets.'
        logging.getLogger(__name__).warning(msg)
        return None

Event = namedtuple('Event', ('name', 'old_value', 'new_value'))


class Service(object):
    possible_events = set()

    def __init__(self,
                 type, id, alias,
                 device):
        self.id = id
        self.type = type
        self.alias = alias
        self.refresh_freq = 0.0
        self._update_time = 0.01
        self._delegate = device
        self._value = None
        self._cb = defaultdict(list)
        self._led = False
        self._node_temperature = None
        self._node_voltage = None
        self._firmware_revision = "Unknown"
        self._luos_revision = "Unknown"
        self._robus_revision = "Unknown"
        self._uuid = [0, 0, 0]
        self._killed = False
        self._last_update = time.time()
        self._luos_statistics = {}

    def __repr__(self):
        return ('<{self.type} '
                'alias="{self.alias}" '
                'id={self.id}>'.format(self=self))

    def _update(self, new_state):
        if ((time.time() - self._last_update) != 0):
            self.refresh_freq = ((200.0 * self.refresh_freq) + (1.0 / (time.time() - self._last_update))) / 201.0
            self._last_update = time.time()
        if 'revision' in new_state:
            self._firmware_revision = new_state['revision']
        if 'luos_revision' in new_state:
            self._luos_revision = new_state['luos_revision']
        if 'uuid' in new_state:
            self._uuid = new_state['uuid']
        if 'luos_statistics' in new_state:
            self._luos_statistics = new_state['luos_statistics']

    def _kill(self):
        self._killed = True
        print ("service", self.alias, "have been excluded from the network due to no responses.")

    def _push_value(self, key, new_val):
        if (self._killed) :
            print("service", self.alias,"is excluded.")
        else :
            if isinstance(new_val, float) :
                self._delegate.update_cmd(self.alias, key, float(str("%.3f" % new_val)))
            else :
                self._delegate.update_cmd(self.alias, key, new_val)

    def _push_data(self, key, new_val, data):
        if (self._killed) :
            print("service", self.alias,"is excluded.")
        else :
            self._delegate.update_data(self.alias, key, new_val, data)

    @property
    def firmware_revision(self):
        self._push_value('revision', "")
        time.sleep(0.03)
        return self._firmware_revision

    @property
    def luos_revision(self):
        self._push_value('luos_revision', "")
        time.sleep(0.03)
        return self._luos_revision

    @property
    def uuid(self):
        self._push_value('uuid', "")
        time.sleep(0.03)
        return self._uuid

    @property
    def luos_statistics(self):
        self._push_value('luos_statistics', "")
        time.sleep(0.3)
        try:
            max_table = [self._luos_statistics["rx_msg_stack"], self._luos_statistics["luos_stack"], self._luos_statistics["tx_msg_stack"], self._luos_statistics["buffer_occupation"]]
            max_val = max(max_table)
            s = self.alias + " statistics :"
            s = s + "\n.luos allocated RAM occupation \t= " + repr(max_val)
            s = s + "%\n\t.RX message stack \t  = " + repr(self._luos_statistics["rx_msg_stack"])
            s = s + "%\n\t.TX message stack \t  = " + repr(self._luos_statistics["tx_msg_stack"])
            s = s + "%\n\t.Luos stack \t\t  = " + repr(self._luos_statistics["luos_stack"])
            s = s + "%\n\t.Buffer occupation \t  = " + repr(self._luos_statistics["buffer_occupation"])
            s = s + "%\n.Dropped messages number \t= " + repr(self._luos_statistics["msg_drop"])
            s = s + "\n.Max luos loop delay \t\t= " + repr(self._luos_statistics["loop_ms"])
            s = s + "ms\n.msg max retry number \t\t= " + repr(self._luos_statistics["max_retry"])
            s = s + "\n"
            print(s)
        except:
            print(self.alias + " statistics collection failed.\n")

    def rename(self, name):
        # check if the string start with a number before sending
        self._push_value('rename', name)
        self.alias = name

    @property
    def update_time(self):
        return self._update_time

    @update_time.setter
    def update_time(self, time):
        self._push_value('update_time', time)
        self._update_time= time

    # Events cb handling

    def add_callback(self, event, cb):
        if event not in self.possible_events:
            raise ValueError('Unknown callback: {} (poss={})'.format(event, self.possible_events))

        self._cb[event].append(cb)

    def remove_callback(self, event, cb):
        self._cb[event].remove(cb)

    def _pub_event(self, trigger, old_value, new_value):
        event = Event(trigger, old_value, new_value)

        for cb in self._cb[trigger]:
            cb(event)
